/*
 * context.h - context routines
 *
 * This file is a part of the multithreading version of the
 * SimpleScalar tool suite written by Todd M. Austin as a part
 * of the Multiscalar Research Project.
 * 
 * Copyright (C) 1997,1998,1999 by Dominik Madon
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dominik.madon@epfl.ch
 * Snail: LSL-DI/EPFL, In-Ecublens, 1015 Lausanne
 *
 * $Id: context.h,v 1.3 2002/01/08 02:13:29 yeung Exp dongkeun $
 *
 *
 */

#ifndef CONTEXT_H
#define CONTEXT_H

#include <stdio.h>
#include "ss.h"

extern SS_CONTEXT_TYPE context_num;
extern int context_config[SS_NUM_CONTEXT];
extern int context_nelt;
extern int context_master[SS_NUM_CONTEXT];

typedef struct msg_port_t {
  SS_WORD_TYPE port;            /* port number */
  SS_WORD_TYPE value;           /* message */
  SS_CONTEXT_TYPE sender_id;    /* sender's context id */
  SS_CONTEXT_TYPE receiver_id;  /* reader's context id */
  unsigned int read;            /* true if msg port was read */
  unsigned int wrote;           /* true if msg was written */
  unsigned int done;            /* true if communication was completed */
} MSG_PORT_TYPE;

extern struct msg_port_t msg_data[SS_NUM_CONTEXT];

void
msg_port_init(void);

unsigned int msg_port_write(
  SS_CONTEXT_TYPE context,
  SS_WORD_TYPE port,
  SS_WORD_TYPE value,
  SS_CONTEXT_TYPE * receiver_context);

unsigned int msg_port_read(
  SS_CONTEXT_TYPE context,
  SS_WORD_TYPE port,
  SS_WORD_TYPE * value,
  SS_CONTEXT_TYPE * sender_context);

unsigned int port_used_by_context(SS_CONTEXT_TYPE context);
SS_CONTEXT_TYPE context_using_port(SS_WORD_TYPE port);
void msg_port_delete(SS_CONTEXT_TYPE context);
unsigned int msg_port_test(SS_WORD_TYPE port);


typedef struct lock_port_t {
  unsigned int free;            /* set if lock is free */
  SS_WORD_TYPE port;            /* port number */
  SS_WORD_TYPE next_port;       /* next port number to use */
  SS_CONTEXT_TYPE context;      /* lock's context id */
} LOCK_PORT_TYPE;

extern struct lock_port_t lock_data[2 * SS_NUM_CONTEXT];

void lock_port_init(void);

unsigned int lock_port_lock(
  SS_CONTEXT_TYPE context, 
  SS_WORD_TYPE lock_id, 
  SS_WORD_TYPE value);
unsigned int lock_port_unlock(
  SS_CONTEXT_TYPE context, 
  SS_WORD_TYPE lock_id, 
  SS_CONTEXT_TYPE * next_context);


/* context_changed() return values */

#define CTXT_CHNG_NONE          0
#define CTXT_CHNG_WAKEUP        1
#define CTXT_CHNG_FREEZE        2

unsigned int context_changed(
  SS_WORD_TYPE old_reg,
  SS_WORD_TYPE new_reg,
  SS_CONTEXT_TYPE *context);


/* PC backup list */

typedef struct rbackup_t {
  SS_WORD_TYPE regs_PC;
} RBACKUP_TYPE;

extern struct rbackup_t rbackup_data[SS_NUM_CONTEXT];

/* Semaphore hardware */

typedef struct semaphore_t_ {
  int value;
  int num_blocked;
  int head;
  int tail;
  int blocked_ID[SS_NUM_CONTEXT];
} semaphore_t;

extern semaphore_t semaphores[SS_NUM_CONTEXT];

void semaphore_init(void);
void semaphore_set(int context, int value);
void semaphore_p(int index);
void semaphore_v(int index);
void semaphore_v_suspend(int index);

#endif /* CONTEXT_H */


