/*
 * interrupt.h - External interruptions implementation
 *
 * This file is a part of the multithreading version of the
 * SimpleScalar tool suite written by Todd M. Austin as a part
 * of the Multiscalar Research Project.
 * 
 * Copyright (C) 1997, 1998, 1999 by Dominik Madon
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dominik.madon@epfl.ch
 * Snail: LSL-DI/EPFL, In-Ecublens, 1015 Lausanne
 *
 * $Id: interrupt.h,v 1.1 1999/08/02 06:36:15 madon Exp $
 *
 *
 */


#ifndef INTERRUPT_H
#define INTERRUPT_H

#include "ss.h"

/* FIXMEDM: cycle are long int !!! */

/* Type of the INTERRUPT */
typedef struct interrupt_t {
  unsigned long int next_interrupt;
  unsigned long int repetition;
  unsigned int port;
} INTERRUPT_TYPE;



INTERRUPT_TYPE * interrupt_init(void);

void interrupt_free(INTERRUPT_TYPE *interrupt_db);

void interrupt_setconfig(
  unsigned long int first_cycle,
  unsigned long int repetition,
  unsigned int port,
  INTERRUPT_TYPE *interrupt_db);

unsigned long int interrupt_next_cycle(
  long unsigned int cycle,
  INTERRUPT_TYPE *interrupt_db);

unsigned int interrupt_port(
  INTERRUPT_TYPE *interrupt_db);


#endif /* INTERRUPT_H */
