/*
 * RCS:
 *     $Source: /projects/vortex/sim/vsim/RCS/vortex.h,v $
 *     $Author: sliknick $
 *     $Date: 2000/03/01 21:24:31 $
 *     $Revision: 1.5 $
 * 
 * vortex.h
 * 
   C macros for special instructions used by the Vortex memory
   controller.
 *
 * Donald Yeung
 * 17 Feb 99
 */

#ifndef VORTEX_H
#define VORTEX_H

#define NOREORDER(cmd) {\
  asm (".set	noreorder");\
  cmd;\
  asm (".set	reorder");\
}

#define VSIM_MAX_STREAM_DESC 2048
#define VSIM_MAX_DIM    16
#define VSIM_MAX_LEVEL  24

/* vsim functional simulation enter/exit macros */
#define VSIM_ENTER NOREORDER(asm volatile ("nop/2:0(1)"));
#define VSIM_EXIT NOREORDER(asm volatile ("nop/2:0(2)"));
#define VSIM_DEBUG_ENTER asm ("nop/2:0(3)");
#define VSIM_DEBUG_EXIT asm ("nop/2:0(4)");

/* DK: set or reset the perfect memory mode */
#define SET_PERF NOREORDER(asm volatile ("nop/d"));
#define RESET_PERF NOREORDER(asm volatile ("nop/e"));

/* DY:  markers that print in commit for debugging */
#define MARK1 NOREORDER(asm ("addiu/2:0(1) $0, $0, 0"));
#define MARK2 NOREORDER(asm ("addiu/2:0(2) $0, $0, 0"));
#define VSIM_COUNTER NOREORDER(asm ("addiu/2:0(3) $0, $0, 0"));
#define SIM_PRINT(X) NOREORDER(asm volatile ("addiu/2:0(5) $0, %0, 0" : : "r" (X)));

/* DK:  pointers to measure time in between */
#define POINT1 NOREORDER(asm ("addiu/5:3(1) $0, $0, 0"));
#define POINT2 NOREORDER(asm ("addiu/5:3(2) $0, $0, 0"));
#define POINT3 NOREORDER(asm ("addiu/5:3(3) $0, $0, 0"));
#define POINT4 NOREORDER(asm ("addiu/5:3(4) $0, $0, 0"));

#define SEM_S(context, value) NOREORDER(asm volatile ("sems %0, %1" : : "r" (context), "r" (value)));
#define SEM_P(X) NOREORDER(asm volatile ("semp $0, %0" : : "r" (X)));
#define SEM_V(X) NOREORDER(asm volatile ("semv $0, %0" : : "r" (X)));
#define SEM_V_SUSPEND(X) NOREORDER(asm volatile ("semvb $0, %0" : : "r" (X)));

#define prefetch(x) NOREORDER(asm volatile ("lb/2:0(1)    $0,0(%0)" : : "r" (x)));
#define blocking(x) NOREORDER(asm volatile ("lb	$0,0(%0)" : : "r" (x)));
#define touching(X) NOREORDER(asm volatile ("addiu	$0, %0, 0" : : "r" (X)));

#define VSIM_GET_CLOCK(x) asm ("add.d/a   %0,$f0,$f0" : "=f" (x) : );

/* stream initialization macros annotated as S where S is stream num */ 
#define VSIM_INIT_STREAM(S) NOREORDER(asm volatile ("nop/15:4("#S")/2:0(5)"));

#define SYNC_LABEL(Lab) NOREORDER(asm volatile ("$LC200"#Lab":"))
#define GET_SYNC_ADDR(X, Lab) asm volatile ("la %0,$LC200"#Lab : "=r" (X) : );
#define SSMT_EXIT() NOREORDER(asm volatile ("spcs/a $0, $0")); 

/* stream synchronization macros annotated as S where S is stream num */ 
#define VSIM_SYNC_STREAM(S,D) \
  NOREORDER(asm volatile ("lw/15:4("#S")/2:0(2)    $0,%0($0)" : :"i" (D)));

/* stream sankronization macros annotated as S where S is stream num */ 
#define VSIM_SANK_STREAM(S,D)  \
  NOREORDER(asm volatile ("lw/15:4("#S")/2:0(3)    $0,%0($0)" : :"i" (D)));

/* stop prefetching and deallocate the stream S */
#define VSIM_STOP_STREAM(S) NOREORDER(asm volatile ("nop/15:4("#S")/2:0(6)"));

/* data structures for stream descriptor */
typedef struct _dim_t
{
  /* parent dim info */
  unsigned A;
  unsigned B;
  unsigned BASE_IND;
  unsigned parent;
  unsigned recurse_parent;

  /* info that specs addresses within dim */
  unsigned LENGTH;
  unsigned STRIDE;
  unsigned IND;

  unsigned pref_dist;  /* prefetch distance */
} dim_t;

typedef struct _stream_t
{
    int n_dim;
    unsigned base_addr;
    dim_t dims[VSIM_MAX_DIM];
} vortexStream_t;

/* memory mappings for stream descriptor locations */
#define VSIM_SEGMENT_BASE      0x00010000

#define STREAM_DESC_BASE_ADDR  0x00010000
#define STREAM_DESC_MAX_SIZE   0x400
#define STREAM_DESC_ADDR(S)   (STREAM_DESC_BASE_ADDR + S * STREAM_DESC_MAX_SIZE)

#endif /* VORTEX_H */

