/*
 * ssmt-2.h
 *
 * Dominik Madon, LSL/DI/EPFL, 1998
 *
 * This file contains definitions for use of ssmt's
 * "context port" functional unit.
 *
 * This file is part of my thesis work.
 *
 */


#ifndef SSMT_4_H
#define SSMT_4_H

#define SS_NUM_CONTEXT 32

typedef int SS_WORD_TYPE;
typedef short int SS_CONTEXT_TYPE;


void readPort(SS_WORD_TYPE *data, SS_WORD_TYPE port);
void writePort(SS_WORD_TYPE data, SS_WORD_TYPE port);

void loadMMUconfig(SS_WORD_TYPE config, SS_CONTEXT_TYPE context);
SS_WORD_TYPE getMMUconfig(SS_CONTEXT_TYPE context);

void loadContextUnitReg(SS_WORD_TYPE data, SS_WORD_TYPE reg);
SS_WORD_TYPE getContextUnitReg(SS_WORD_TYPE reg);

void putMessage(SS_WORD_TYPE data, SS_WORD_TYPE port);
#define getMessage(port, value) \
  NOREORDER(asm volatile ("gmsg %0, %1" : "=r" (value) : "r" (port)));

void lock(SS_WORD_TYPE next_port, SS_WORD_TYPE lock);
SS_WORD_TYPE unlock(SS_WORD_TYPE lock);

void loadPCandResume(SS_WORD_TYPE pc, SS_WORD_TYPE context);
SS_WORD_TYPE getPCandStop(SS_WORD_TYPE context);

#define createThread(threadId, context, stack, func, params) \
  createThread_(threadId, context, stack, func, params, 0x1000);

#define createThread_(threadId, context, stack, func, params, port) \
  ((unsigned *)stack)[-7] = (unsigned)threadId; \
  ((unsigned *)stack)[-6] = (unsigned)params; \
  ((unsigned *)stack)[-5] = (unsigned)func; \
  NOREORDER(asm volatile ("sw $28, -16(%0)" : : "r" (stack))); \
  NOREORDER(asm volatile ("lpcr %0, %1" : : "r" (_startThread), "r" (context))); \
  NOREORDER(asm volatile ("pmsg %0, %1" : : "r" (stack), "r" (port)));

#define getRegACXT(value) \
  NOREORDER(asm volatile ("scu %0, $0" : "=r" (value)));

#define getCXT(value) \
  NOREORDER(asm volatile ("scu/a %0, $0" : "=r" (value)));

#define setSP() \
  NOREORDER(asm volatile ("scu/b $0, $0"));

#define resumeContext(value) \
{ \
  unsigned cxt_PC; \
  GET_SYNC_ADDR(cxt_PC, resume_label); \
  NOREORDER(asm volatile ("lpcr/a %0, %1" : : "r" (cxt_PC), "r" (value))); \
}

#define resumeThread(value) \
{ \
  unsigned cxt_PC; \
  NOREORDER(asm volatile ("spcs %0, %1" : "=r" (cxt_PC) : "r" (value))); \
  NOREORDER(asm volatile ("lpcr %0, %1" : : "r" (cxt_PC), "r" (value))); \
}

#define suspendContext(value) \
  NOREORDER(asm volatile ("spcs $0, %0" : : "r" (value)));


int waitThread(SS_WORD_TYPE threadId);

int activeContext_p(SS_CONTEXT_TYPE context);

int swapThreadOut(
      SS_CONTEXT_TYPE context, 
      SS_WORD_TYPE *sp, 
      SS_WORD_TYPE *pc);

int swapThreadIn(
      SS_CONTEXT_TYPE context,
      SS_WORD_TYPE sp,
      SS_WORD_TYPE pc);

extern void _startThread();

#endif
