# ssmt-3.s
#
# Implementation part of the routines to exploit the
# context unit related functions.       
#
# This implementation uses extended instructions.
#                
# Dominik Madon, LSL-EPFL, 1998, 1999, 2000.        
# This file is part of my thesis work.

        .align  2
        .globl  loadMMUconfig
        .align  2
        .globl  getMMUconfig
        .align  2
        .globl  loadContextUnitReg     
        .align  2
        .globl  getContextUnitReg     
        .align  2
        .globl  putMessage
        .align  2
        .globl  getMessage
        .align  2
        .globl  lock
        .align  2
        .globl  unlock
        .align  2
        .globl  loadPCandResume
        .align  2
        .globl  getPCandStop
        
       	.align	2
	.globl	createThread
	.align	2
	.globl	waitThread

       	.align	2
       	.globl	activeContext_p
	.align	2
	.globl	suspendContext
	.align	2
	.globl	resumeContext

       	.align	2
	.globl	writePort
	.align	2
	.globl	readPort


        .align	2
        .globl	swapThreadOut
        .align	2
        .globl	swapThreadIn

        
        .align	2
        .globl	_startThread
        .align	2
        .globl	_swapContextOut
        .align	2
        .globl	_swapContextIn
        
	.text




#
# l o a d M M U c o n f i g 
#
# This function set MMU configuration for a given context.
#
# Parameters
#   $4 value
#   $5 context
#                                
# WARING: There is no bound check !
#        

        .ent    loadMMUconfig
loadMMUconfig:
        lmmu    $4,$5
        jr      $31
        .end    loadMMUconfig        

        



#
# g e t M M U c o n f i g 
#
# This function reads MMU configuration of a given context.
#
# Parameters
#   $4 context
#
# Returns MMU configuration value
#        
# WARING: There is no bound check !
#        

        .ent    getMMUconfig
getMMUconfig:
        smmu    $2,$4
        jr      $31
        .end    getMMUconfig        




# 
# l o a d C o n t e x t U n i t R e g
#
# Parameters
#               
#     $4 data
#     $5 register

        .ent    loadContextUnitReg
loadContextUnitReg:
        lcu     $4,$5
        jr      $31
        .end    loadContextUnitReg
  
              


#        
# g e t C o n t e x t U n i t R e g
#
# Parameters
#   $4 register
# Return
#   $2 value
#                
        .ent    getContextUnitReg
getContextUnitReg:
        scu     $2,$4
	j	$31
        .end    getContextUnitReg



#
# p u t M e s s a g e 
#        
# Parameters       
#   $4 message
#   $5 port               
#
putMessage:     
        .ent    putMessage
        pmsg    $4,$5
	j	$31
        .end    putMessage                       
        
        
        
# DY:	This is now a macro
# g e t M e s s a g e 
#        
# Parameters       
#   $4 port
# Return
#   $2 value
#
	
#getMessage:     
#        .ent    getMessage
#        gmsg    $2,$4
#	j	$31
#        .end    getMessage                       

        
        
#        
# l o c k       
#        
# Parameters       
#    $4 port    
#    $5 lock
#
lock:   
        .ent    lock
        lck    $4,$5
	j	$31
        .end    lock
        
               
        
#        
# u n l o c k       
#        
# Parameters       
#    $4 lock
# Return        
#    $2 1 if a context was resumed
#
unlock:   
        .ent    unlock
        ulck    $2,$4
	j	$31
        .end    unlock
        
               

#        
# l o a d P C a n d R e s u m e
#        
# Parameters
#    $4 pc
#    $5 context
#
loadPCandResume:        
        .ent    loadPCandResume
        lpcr    $4,$5
	j	$31        
        .end    loadPCandResume                                

        
        
#        
# g e t P C a n d S t o p
#        
# Parameters
#    $4 context
# Return        
#    $2 pc
#
getPCandStop:        
        .ent    getPCandStop
        spcs    $2,$4
	j	$31        
        .end    getPCandStop

        
        
#
# _ s t a r t T h r e a d
#
# This function is called by a new created thread.
#
     
_startThread:
        .ent    _startThread
        li      $sp,0x1000
        gmsg    $sp,$sp                 
        lw      $4,-28($sp)               # threadID
        lw      $5,-24($sp)               # argv
        lw      $3,-20($sp)              # function to execute
        lw      $28,-16($sp)             # gp register
        subu    $sp,$sp,48
        jalr    $31,$3
_sT1:   lw      $3,20($sp)
	addu    $3,$3,0x1000            # termination port
#        pmsg    $2,$3                   # function return
        lcu     $0,$0                   # endless wait
        .end    _startThread
        
                                
                
# DY:	This is now a macro
# c r e a t e T h r e a d
# 
# Parameters
#   $4 threadId
#   $5 context
#   $6 (stack)
#   $7 (function to execute)
#      (function parameter)
#
# context + 0x1000 is returned
# 

#       	.ent	createThread
#createThread:
#        sw      $4,-28($6)              # threadID
#        lw      $2,16($sp)              # fifth parameter
#        sw      $2,-24($6)              # argv
#        sw      $7,-20($6)              # function to execute   
#        sw      $28,-16($6)             # r28 will be used by the child
#        la      $2,_startThread
#        lpcr    $2,$5                   # writing new PC
#        li      $2,0x1000
#        pmsg    $6,$2                   # transmiting thread's stack pointer
#	j	$31
#	.end	createThread


        
#        
# w a i t T h r e a d
#
# Parameters
#   $4 threadId
#
# return value returned by the context
#        
                        
	.ent	waitThread
waitThread:
        addu    $2,$4,0x1000
        gmsg    $2,$2                   # return value
      	j	$31
	.end	waitThread



       


#        
# a c t i v e C o n t e x t _ p
#
# tests if the given context is running
#                      
# parameter:    
#   $4 context number
#
# returns 0 if the context is not running
#
        
        .ent    activeContext_p
activeContext_p:
	subu	$sp,$sp,32
	sw	$31,8($sp)
	sw	$fp,4($sp)
	move	$fp,$sp
        sw      $5,12($fp)
        sw      $6,16($fp)
        scu     $2,$0
        li      $6,0x1
        sll     $5,$6,$4
        and     $2,$2,$5
        lw      $5,12($fp)
        lw      $6,16($fp)
	move	$sp,$fp			# sp not trusted here
	lw	$31,8($sp)
	lw	$fp,4($sp)
	addu	$sp,$sp,32
	j	$31
        .end    activeContext_p



        
# DY:	This is now a macro
# s u s p e n d C o n t e x t
#
# stop execution of the given context.
#
# parameters:   
#   $4 context to stop
#
# return value: 
#   $2 PC of the suspended context
#                        
	
#       	.ent	suspendContext
#suspendContext:
#        spcs    $2,$4
#	j	$31
#	.end	suspendContext



        
# DY:	This is now a macro
# r e s u m e C o n t e x t
#
# resume execution of the given context.
#
# parameters:   
#   $4 context to resume
#
# return value: 
#   $2 PC of the resumed context
#                        
	
#       	.ent	resumeContext
#resumeContext:
#        spcs    $2,$4
#        lpcr    $2,$4
#	j	$31
#	.end	resumeContext



        

# w r i t e P o r t 
#
# send a word data to a given port.
#
# parameters:
#   $4 data to send
#   $5 port number                
#
        .ent writePort
writePort:  
        wport   $4,$5
	j	$31
        .end writePort


        
#        
# r e a d P o r t
#
# receive a word data from a given port.
#
# parameters:
#   $4 (data read)
#   $5 port number
#
        .ent readPort
readPort:  
	subu	$sp,$sp,32
	sw	$31,8($sp)
	sw	$fp,4($sp)
	move	$fp,$sp
        sw      $6,16($fp)
        rport   $6,$5
        sw      $6,($4)
        lw      $6,16($fp)
	move	$sp,$fp			# sp not trusted here
	lw	$31,8($sp)
	lw	$fp,4($sp)
	addu	$sp,$sp,32
	j	$31
        .end readPort




#
# _ c o n t e x t O u t
#
# This procedure is called by swapThreadOut
#        

        .ent    _contextOut
_contextOut:                     
        .set noat       
        sw     $0,-256($sp)
        sw     $1,-252($sp)
        sw     $2,-248($sp)
        sw     $3,-244($sp)
        sw     $4,-240($sp)
        sw     $5,-236($sp)
        sw     $6,-232($sp)
        sw     $7,-228($sp)
        sw     $8,-224($sp)
        sw     $9,-220($sp)
        sw     $10,-216($sp)
        sw     $11,-212($sp)
        sw     $12,-208($sp)
        sw     $13,-204($sp)
        sw     $14,-200($sp)
        sw     $15,-196($sp)
        sw     $16,-192($sp)
        sw     $17,-188($sp)
        sw     $18,-184($sp)
        sw     $19,-180($sp)
        sw     $20,-176($sp)
        sw     $21,-172($sp)
        sw     $22,-168($sp)
        sw     $23,-164($sp)
        sw     $24,-160($sp)
        sw     $25,-156($sp)
        sw     $26,-152($sp)
        sw     $27,-148($sp)
        sw     $28,-144($sp)
        sw     $30,-136($sp)
        sw     $31,-132($sp)
        mflo    $6
        mfhi    $7
        dsw     $6,-264($sp)
        add     $6,$0,$0
        bc1f    cO_1
        li      $6,1
cO_1:   sw      $6,-272($sp)
        s.d     $f0,-128($sp)
        s.d     $f2,-120($sp)
        s.d     $f4,-112($sp)
        s.d     $f6,-104($sp)
        s.d     $f8,-96($sp)
        s.d     $f10,-88($sp)
        s.d     $f12,-80($sp)
        s.d     $f14,-72($sp)
        s.d     $f16,-64($sp)
        s.d     $f18,-56($sp)
        s.d     $f20,-48($sp)
        s.d     $f22,-40($sp)
        s.d     $f24,-32($sp)
        s.d     $f26,-24($sp)
        s.d     $f28,-16($sp)
        s.d     $f30,-8($sp)
        li      $4,0x2000       # swap port
        pmsg    $sp,$4
        lcu     $0,$0           # context stopped
        .end    _contextOut


        
#
# s w a p T h r e a d O u t
#
# Makes a context to dump all its regsiters on its stack and stop.
#
# FIXMEDM:      
#   This function cannot be used for stopping several contexts
#   unless stops serialized.
#        
# parameters:   
#    $4 context
#    $5 (sp)
#    $6 (pc)        
#
        .ent swapThreadOut
swapThreadOut:
        spcs    $2,$4                   # reading PC/stopping
        sw      $2,0($6)                # store PC value
        la      $2,_contextOut
        lpcr    $2,$4                   # swap out routine
        li      $2,0x2000
        gmsg    $2,$2                   # reading thread's stack pointer
        sw      $2,0($5)
        j	$31
        .end swapThreadOut

                




#
# _ c o n t e x t I n
#
# This procedure is called by swapThreadIn
#        

        .ent    _contextIn
        .set noat
_contextIn:                     
        li      $4,0x2001       # swapin port
        gmsg    $sp,$4
        dlw     $6,-264($sp)
        mtlo    $6
        mthi    $7
        lw      $6,-272($sp)
        mtc1    $0,$f0
        mtc1    $6,$f2
        c.eq.s  $f0,$f2
        l.d     $f0,-128($sp)
        l.d     $f2,-120($sp)
        l.d     $f4,-112($sp)
        l.d     $f6,-104($sp)
        l.d     $f8,-96($sp)
        l.d     $f10,-88($sp)
        l.d     $f12,-80($sp)
        l.d     $f14,-72($sp)
        l.d     $f16,-64($sp)
        l.d     $f18,-56($sp)
        l.d     $f20,-48($sp)
        l.d     $f22,-40($sp)
        l.d     $f24,-32($sp)
        l.d     $f26,-24($sp)
        l.d     $f28,-16($sp)
        l.d     $f30,-8($sp)
        lw      $0,-256($sp)
        lw      $1,-252($sp)
        lw      $2,-248($sp)
        lw      $3,-244($sp)
        lw      $4,-240($sp)
        lw      $5,-236($sp)
        lw      $6,-232($sp)
        lw      $7,-228($sp)
        lw      $8,-224($sp)
        lw      $9,-220($sp)
        lw      $10,-216($sp)
        lw      $11,-212($sp)
        lw      $12,-208($sp)
        lw      $13,-204($sp)
        lw      $14,-200($sp)
        lw      $15,-196($sp)
        lw      $16,-192($sp)
        lw      $17,-188($sp)
        lw      $18,-184($sp)
        lw      $19,-180($sp)
        lw      $20,-176($sp)
        lw      $21,-172($sp)
        lw      $22,-168($sp)
        lw      $23,-164($sp)
        lw      $24,-160($sp)
        lw      $25,-156($sp)
        lw      $26,-152($sp)
        lw      $27,-148($sp)
        lw      $28,-144($sp)
        li      $31,0x2001
        lw      $30,-136($sp)
        pmsg    $sp,$31         # send message to creator
        lw      $31,-132($sp)
        lcu     $0,$0           # context stopped
        .end    _contextIn
                
        
#
# s w a p T h r e a d I n
#
# Makes a context to load thread.
#
# parameters:   
#    $4 context
#    $5 sp
#    $6 pc        
#

        
        .ent swapThreadIn
swapThreadIn:  
	subu	$sp,$sp,32
	sw	$8,8($sp)
	sw	$9,12($sp)
        la      $2,_contextIn
        lpcr    $2,$4                   # swap in routine
        li      $2,0x2001
        pmsg    $5,$2                   # sending sp
        gmsg    $2,$2                   # waiting for semicompletion...
_sTI:   scu     $2,$0          
        li      $9,0x1
        sll     $8,$9,$4
        and     $2,$2,$8
        bne     $0,$2,_sTI                
        lpcr    $6,$4                   # here the context is stopped
	lw	$8,8($sp)
	lw	$9,12($sp)
	addu	$sp,$sp,32
	j	$31
        .end swapThreadIn





